@@ -2,48 +2,40 @@ import datetime
 import sys
 import logging
 import traceback
-import warnings
-from typing import Optional, List
+from typing import Optional, List, Any, Dict, Union, Callable
 
 import humanize
 
 from bot_base.caches import TimedCache
 
 try:
-    import nextcord as discord
-    from nextcord import DiscordException
+    import nextcord
+    from nextcord import DiscordException, abc
     from nextcord.ext import commands
+    from nextcord.ext.commands.converter import CONVERTER_MAPPING
 except ModuleNotFoundError:
-    import discord
-    from discord import DiscordException
-    from discord.ext import commands
+    import disnake as nextcord
+    from disnake import DiscordException, abc
+    from disnake.ext import commands
+    from disnake.ext.commands.converter import CONVERTER_MAPPING
 
 from bot_base.blacklist import BlacklistManager
 from bot_base.context import BotContext
 from bot_base.db import MongoManager
-from bot_base.exceptions import PrefixNotFound
+from bot_base.exceptions import PrefixNotFound, BlacklistedEntry
 from bot_base.wraps import (
     WrappedChannel,
     WrappedMember,
-    WrappedChannelConvertor,
-    WrappedMemberConvertor,
-    WrappedUserConvertor,
     WrappedUser,
+    WrappedThread,
 )
 
 log = logging.getLogger(__name__)
 
-try:
-    from nextcord.ext.commands.converter import CONVERTER_MAPPING
 
-    CONVERTER_MAPPING[discord.User] = WrappedUserConvertor
-    CONVERTER_MAPPING[discord.Member] = WrappedMemberConvertor
-    CONVERTER_MAPPING[discord.TextChannel] = WrappedChannelConvertor
-except ModuleNotFoundError:
-    warnings.warn(
-        "You don't have overridden converters. "
-        "Please open an issue and name the fork your using if you want them."
-    )
+CONVERTER_MAPPING[nextcord.User] = WrappedUser
+CONVERTER_MAPPING[nextcord.Member] = WrappedMember
+CONVERTER_MAPPING[nextcord.TextChannel] = WrappedChannel
 
 
 class BotBase(commands.Bot):
@@ -54,7 +46,16 @@ class BotBase(commands.Bot):
                 kwargs.pop("mongo_url"), kwargs.pop("mongo_database_name", None)
             )
 
-        self.blacklist: BlacklistManager = BlacklistManager(self.db)
+        try:
+            self.blacklist: BlacklistManager = BlacklistManager(self.db)
+        except AttributeError:
+            log.warning(
+                "You do not have a blacklist setup. "
+                "Please set `self.db` to a instance/subclass of MongoManager before "
+                "calling (..., leave_db=True) if you wish to have a blacklist."
+            )
+            self.blacklist = None
+
         self._uptime: datetime.datetime = datetime.datetime.now(
             tz=datetime.timezone.utc
         )
@@ -68,6 +69,17 @@ class BotBase(commands.Bot):
         if kwargs.pop("load_builtin_commands", None):
             self.load_extension("bot_base.cogs.internal")
 
+        # These events do include the on_ prefix
+        self._single_event_type_sheet: Dict[str, Callable] = {
+            "on_message": self.get_wrapped_message,
+        }
+        self._double_event_type_sheet: Dict[str, Callable] = {
+            "on_message_edit": lambda before, after: (
+                self.get_wrapped_message(before),
+                self.get_wrapped_message(after),
+            )
+        }
+
     @property
     def uptime(self) -> datetime.datetime:
         return self._uptime
@@ -78,22 +90,34 @@ class BotBase(commands.Bot):
         )
 
     async def on_ready(self) -> None:
-        await self.blacklist.initialize()
+        if self.blacklist:
+            await self.blacklist.initialize()
 
-    async def get_command_prefix(self, message: discord.Message) -> List[str]:
+    async def get_command_prefix(
+        self, bot: "BotBase", message: nextcord.Message
+    ) -> List[str]:
         try:
             prefix = await self.get_guild_prefix(guild_id=message.guild.id)
 
-            if message.content.casefold().startswith(prefix.casefold()):
-                # The prefix matches, now return the one the user used
-                # such that dpy will dispatch the given command
-                prefix_length = len(prefix)
-                prefix = message.content[:prefix_length]
+            prefix = self.get_case_insensitive_prefix(message.content, prefix)
 
             return commands.when_mentioned_or(prefix)(self, message)
 
         except (AttributeError, PrefixNotFound):
-            return commands.when_mentioned_or(self.DEFAULT_PREFIX)(self, message)
+            prefix = self.get_case_insensitive_prefix(
+                message.content, self.DEFAULT_PREFIX
+            )
+            return commands.when_mentioned_or(prefix)(self, message)
+
+    @staticmethod
+    def get_case_insensitive_prefix(content, prefix):
+        if content.casefold().startswith(prefix.casefold()):
+            # The prefix matches, now return the one the user used
+            # such that dpy will dispatch the given command
+            prefix_length = len(prefix)
+            prefix = content[:prefix_length]
+
+        return prefix
 
     async def get_guild_prefix(self, guild_id: Optional[int] = None) -> str:
         """
@@ -119,7 +143,7 @@ class BotBase(commands.Bot):
         if guild_id in self.prefix_cache:
             return self.prefix_cache.get_entry(guild_id)
 
-        prefix_data = self.db.config.find(guild_id)
+        prefix_data = await self.db.config.find(guild_id)
 
         if not prefix_data:
             raise PrefixNotFound
@@ -140,16 +164,16 @@ class BotBase(commands.Bot):
             await ctx.author.send("Sorry. This command is disabled and cannot be used.")
         elif isinstance(error, commands.CommandInvokeError):
             original = error.original
-            if not isinstance(original, discord.HTTPException):
+            if not isinstance(original, nextcord.HTTPException):
                 print(f"In {ctx.command.qualified_name}:", file=sys.stderr)
                 traceback.print_tb(original.__traceback__)
                 print(f"{original.__class__.__name__}: {original}", file=sys.stderr)
         elif isinstance(error, commands.ArgumentParsingError):
             await ctx.send(error)
-        elif isinstance(error, commands.PrivateMessageOnly):
-            await ctx.send("This command can only be used in dm's.")
-        elif isinstance(error, commands.NoPrivateMessage):
-            await ctx.send("This command can only be used in Guilds.")
+        elif isinstance(error, commands.NotOwner):
+            await ctx.send("You do not have permissions to run this command.")
+        elif isinstance(error, BlacklistedEntry):
+            await ctx.send(error.message)
 
         if not isinstance(error, commands.CommandNotFound):
             if await self.db.command_usage.find(ctx.command.qualified_name) is None:
@@ -186,25 +210,32 @@ class BotBase(commands.Bot):
             )
         log.debug(f"Command executed: `{ctx.command.qualified_name}`")
 
-    async def on_guild_join(self, guild: discord.Guild) -> None:
-        if guild.id in self.blacklist.guilds:
+    async def on_guild_join(self, guild: nextcord.Guild) -> None:
+        if self.blacklist and guild.id in self.blacklist.guilds:
             log.info("Leaving blacklisted Guild(id=%s)", guild.id)
             await guild.leave()
 
-    async def process_commands(self, message: discord.Message) -> None:
+    async def process_commands(self, message: nextcord.Message) -> None:
         ctx = await self.get_context(message, cls=BotContext)
 
-        if ctx.author.id in self.blacklist.users:
+        if self.blacklist and ctx.author.id in self.blacklist.users:
             log.debug(f"Ignoring blacklisted user: {ctx.author.id}")
-            return
+            raise BlacklistedEntry(f"Ignoring blacklisted user: {ctx.author.id}")
 
-        if ctx.guild is not None and ctx.guild.id in self.blacklist.guilds:
+        if self.blacklist and ctx.guild is not None and ctx.guild.id in self.blacklist.guilds:
             log.debug(f"Ignoring blacklisted guild: {ctx.guild.id}")
-            return
+            raise BlacklistedEntry(f"Ignoring blacklisted guild: {ctx.guild.id}")
+
+        if ctx.command:
+            log.debug(
+                "Invoked command %s for User(id=%s)",
+                ctx.command.qualified_name,
+                ctx.author.id,
+            )
 
         await self.invoke(ctx)
 
-    async def on_message(self, message: discord.Message) -> None:
+    async def on_message(self, message: nextcord.Message) -> None:
         if message.author.bot:
             log.debug("Ignoring a message from a bot.")
             return
@@ -216,21 +247,21 @@ class BotBase(commands.Bot):
         guild = await self.get_or_fetch_guild(guild_id)
         member = guild.get_member(member_id)
         if member is not None:
-            return WrappedMember(member)
+            return WrappedMember(member, bot=self)
 
         member = await guild.fetch_member(member_id)
-        return WrappedMember(member)
+        return WrappedMember(member, bot=self)
 
     async def get_or_fetch_channel(self, channel_id: int) -> WrappedChannel:
         """Looks up a channel in cache or fetches if not found."""
         channel = self.get_channel(channel_id)
         if channel:
-            return WrappedChannel(channel)
+            return self.get_wrapped_channel(channel)
 
         channel = await self.fetch_channel(channel_id)
-        return WrappedChannel(channel)
+        return self.get_wrapped_channel(channel)
 
-    async def get_or_fetch_guild(self, guild_id: int) -> discord.Guild:
+    async def get_or_fetch_guild(self, guild_id: int) -> nextcord.Guild:
         """Looks up a guild in cache or fetches if not found."""
         guild = self.get_guild(guild_id)
         if guild:
@@ -243,7 +274,53 @@ class BotBase(commands.Bot):
         """Looks up a user in cache or fetches if not found."""
         user = self.get_user(user_id)
         if user:
-            return WrappedUser(user)
+            return WrappedUser(user, bot=self)
 
         user = await self.fetch_user(user_id)
-        return WrappedUser(user)
+        return WrappedUser(user, bot=self)
+
+    def get_wrapped_channel(
+        self,
+        channel: Union[abc.GuildChannel, abc.PrivateChannel, nextcord.Thread],
+    ) -> Union[WrappedThread, WrappedChannel]:
+        if isinstance(channel, nextcord.Thread):
+            return WrappedThread(channel, self)
+
+        return WrappedChannel(channel, self)
+
+    def get_wrapped_person(
+        self, person: Union[nextcord.User, nextcord.Member]
+    ) -> Union[WrappedUser, WrappedMember]:
+        if isinstance(person, nextcord.Member):
+            return WrappedMember(person, self)
+
+        return WrappedUser(person, self)
+
+    def get_wrapped_message(self, message: nextcord.Message) -> nextcord.Message:
+        """
+        Wrap the relevant params in message with meta classes.
+
+        These fields are:
+        message.channel: Union[WrappedThread, WrappedChannel]
+        message.author: Union[WrappedUser, WrappedMember]
+        """
+        message.channel = self.get_wrapped_channel(message.channel)
+        message.author = self.get_wrapped_person(message.author)
+
+        return message
+
+    def dispatch(self, event_name: str, *args: Any, **kwargs: Any) -> None:
+        _name = f"on_{event_name}"
+        # If we know the event, dispatch the wrapped one
+        if _name in self._single_event_type_sheet:
+            wrapped_arg = self._single_event_type_sheet[_name](args[0])
+            super().dispatch(event_name, wrapped_arg)  # type: ignore
+
+        elif _name in self._double_event_type_sheet:
+            wrapped_first_arg, wrapped_second_arg = self._double_event_type_sheet[
+                _name
+            ](args[0], args[1])
+            super().dispatch(event_name, wrapped_first_arg, wrapped_second_arg, self)
+
+        else:
+            super().dispatch(event_name, *args, **kwargs)  # type: ignore
