@@ -1,3 +1,4 @@
+from copy import deepcopy
 from datetime import timedelta, datetime
 from typing import Any, Dict
 
@@ -6,6 +7,7 @@ from bot_base.caches.abc import Cache
 from bot_base.exceptions import NonExistentEntry, ExistingEntry
 
 
+# This class is unit-tested in my AntiSpam repo
 class TimedCache(Cache):
     __slots__ = ("cache",)
 
@@ -15,7 +17,8 @@ class TimedCache(Cache):
     def __contains__(self, item: Any) -> bool:
         try:
             entry = self.cache[item]
-            if entry.expiry_time < datetime.now():
+            if entry.expiry_time and entry.expiry_time < datetime.now():
+                self.delete_entry(item)
                 return False
         except KeyError:
             return False
@@ -25,11 +28,13 @@ class TimedCache(Cache):
     def add_entry(
         self, key: Any, value: Any, *, ttl: timedelta = None, override: bool = False
     ) -> None:
-        ttl = ttl or timedelta()
         if key in self and not override:
             raise ExistingEntry
 
-        self.cache[key] = Entry(value=value, expiry_time=(datetime.now() + ttl))
+        if ttl:
+            self.cache[key] = Entry(value=value, expiry_time=(datetime.now() + ttl))
+        else:
+            self.cache[key] = Entry(value=value)
 
     def delete_entry(self, key: Any) -> None:
         try:
@@ -45,6 +50,6 @@ class TimedCache(Cache):
 
     def force_clean(self) -> None:
         now = datetime.now()
-        for k, v in self.cache.items():
-            if v.expiry_time < now:
+        for k, v in deepcopy(self.cache).items():
+            if v.expiry_time and v.expiry_time < now:
                 self.delete_entry(k)
