@@ -1,7 +1,7 @@
 try:
     from nextcord import DiscordException
 except ModuleNotFoundError:
-    from discord import DiscordException
+    from disnake import DiscordException
 
 
 class PrefixNotFound(DiscordException):
@@ -14,3 +14,8 @@ class ExistingEntry(DiscordException):
 
 class NonExistentEntry(DiscordException):
     """No entry found in the cache with this key."""
+
+
+class BlacklistedEntry(DiscordException):
+    def __init__(self, message: str):
+        self.message: str = message
